/* ******************************************************************

   class    : AbstractExecute
   date     : 21.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.Component;
import java.awt.Label;
import java.awt.Frame;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Properties;
import java.util.Vector;
import java.util.Observable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import Utils.ResourceUtil;
import Utils.QuotedStringTokenizer;
import Messages.Message;
import Messages.TerminateMessage;
import Messages.ErrorMessage;

/**
 * An abstract base class for plugins that execute files.
 * A derived class can have a method settings(java.awt.Frame)
 * which is automatically recognized by the environment. The
 * purpose of this method is to specify execution settings.
 * A derived class can also have a method
 * selectedExecute(java.aw.Frame). The purpose of this method
 * is to select a file and execute it, if multiple executable
 * files are available.
 *
 * @author puehringer edgar
 */
public abstract class AbstractExecute extends Observable {

  // instance variable declarations
  protected final ResourceBundle resources;
  protected final Properties project;

  // constructors
  /** Create a new AbstractExecute class. */
  public AbstractExecute(ResourceBundle resources, Properties project) {
    this.resources = resources;
    this.project   = project;
  } // end constructor AbstractExecute

  // class methods
  /**
   * Returns an instance of a derived class of AbstractExecute.
   *
   * @param clazz The class name.
   * @param resources Resource bundle containing GUI labels, etc.
   * @param project Project related properties.
   */
  public static AbstractExecute getInstance(String clazz,
                  ResourceBundle resources, Properties project) {
    Object[] objects;
    Class[] classes;
    Class c;
    Constructor constr;
    AbstractExecute execute = null;

    if (clazz == null) return null;
    objects    = new Object[2];
    objects[0] = resources;
    objects[1] = project;
    classes    = new Class[2];
    classes[0] = java.util.ResourceBundle.class;
    classes[1] = java.util.Properties.class;
    try {
      c       = Class.forName(clazz);
      constr  = c.getConstructor(classes);
      execute = (AbstractExecute)constr.newInstance(objects);
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
    } catch (IllegalAccessException excep) {
       excep.printStackTrace();
    } catch (InstantiationException excep) {
       excep.printStackTrace();
    } catch (NoSuchMethodException excep) {
       excep.printStackTrace();
    } catch (InvocationTargetException excep) {
       excep.printStackTrace();
    }
    return execute;
  } // end getInstance

  // methods
  /**
   * Execute a file.
   *
   * @param frame A frame that can be used as parent for dialogs.
   * @see getProjectFiles
   * @see getParameters
   * @see notifyTermination
   */
  public abstract void defaultExecute(Frame frame);

  /**
   * Gets the command line parameters.
   */
  protected String[] getParameters() {
    String str = project.getProperty(Const.CMD_LINE_PARAM);
    QuotedStringTokenizer tok;
    String[] result;

    if (str == null) return new String[0];
    tok    = new QuotedStringTokenizer(str);
    result = new String[tok.countTokens()];
    for (int i=0; i<result.length; i++) {
      result[i] = tok.nextToken();
    } // end for
    return result;
  } // end getParameters

  /**
   * Return copyright information.
   */
  public Component getInfo() {
    return new Label(this.getClass().getName());
  } // end getInfo

  /** Displays a settings dialog. */
  public void settings(Frame parent) {
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      false,"Method should not be called");
  } // end settings

  /**
   * Execute a file selected by the user.
   *
   * @param frame A frame that can be used as parent for dialogs.
   * @see getProjectFiles
   * @see getParameters
   * @see notifyTermination
   */
  public void selectedExecute(Frame parent) {
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      false,"Method should not be called");
  } // end selectedExecute

  /** Returns true, if the settings method is available. */
  public boolean hasSettings() {
    String[] params = new String[1];

    params[0] = "java.awt.Frame";
    return hasMethod("settings",params);
  } // end hasSettings

  /** Returns true, if the selectedExecute method is available. */
  public boolean hasSelectedExecute() {
    String[] params = new String[1];

    params[0] = "java.awt.Frame";
    return hasMethod("selectedExecute",params);
  } // end hasSelectedExecute

  /** Returns true, if the specified method is available. */
  private boolean hasMethod(String method, String[] paramClasses) {
    Class clazz    = this.getClass();
    Class[] params = new Class[paramClasses.length];
    boolean found  = true;
    Method m;

    try {
      for (int i=0; i<params.length; i++)
        params[i] = Class.forName(paramClasses[i]);
      // end for
      m = clazz.getDeclaredMethod(method,params);
    } catch (NoSuchMethodException excep) {
      found = false;
    } catch (SecurityException excep) {
      excep.printStackTrace();
      found = false;
    } catch (ClassNotFoundException excep) {
      excep.printStackTrace();
      found = false;
    }
    return found;
  } // end hasMethod

  /**
   * Returns an array containing all executable project files.
   */
  protected File[] getProjectFiles() {
    int numFiles  = Integer.parseInt(project.getProperty(Const.NUM_EXEC));
    Vector vector = new Vector(numFiles);
    File[] files;

    for (int i=0; i<numFiles; i++) {
      String name = project.getProperty(Const.EXEC_PREFIX + i);

      if (name == null) {
        System.err.println("AbstractExecute: Executable file number wrong.");
        break;
      } // end if
      vector.addElement(new File(ResourceUtil.urlToFile(name)));
    } // end for
    numFiles = vector.size();
    files = new File[numFiles];
    for (int i=0; i<numFiles; i++) files[i] = (File)vector.elementAt(i);
    return files;
  } // end getProjectFiles

  /**
   * Returns an array containing all extensions of executable files.
   */
  protected String[] getExtensions() {
    String str;

    str = project.getProperty(Const.EXEC_EXT);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Extension for executable files not found");
    return ResourceUtil.splitToWords(str);
  } // end getExtensions

  /**
   * Returns the project directory.  It's usefull to add this to
   * the library path (classpath one java platform).
   */
  protected String getProjectDirectory() {
    // This method can both read files and urls
    String str;

    str = project.getProperty(Const.PROJECT_DIR);
    Utils.Assert.assert(Utils.Assert.ENABLED &&
      (str != null),"Project directory not spezified");
    if (str.startsWith("file:/")) str = ResourceUtil.urlToFile(str);
    return str;
  } // end getProjectDirectory

  /** Marks the current project as changed. */
  protected void setProjectChanged() {
    project.put(Const.PROJECT_CHANGED,""+true);
  } // end setProjectChanged

  /** Posts a message to all observers of this observable. */
  public void postMessage(Message msg) {
    setChanged();
    notifyObservers(msg);
  } // end postMessage

  /**
   * Notifies all Observers from terminating a process or thread.
   * The argument given to the Observers is a String that starts
   * with the exit value followed by a blank followed by the
   * absolute path of the executed file.
   */
  protected void notifyTermination(int exitValue, File file) {
    int type;
    String str,path;

    if (file != null) {
      str  = file.getAbsolutePath();
      path = getProjectDirectory();
      if (str.startsWith(path)) {
        str = str.substring(path.length());
        str = str.substring(1);
      } // end if
      type = (exitValue == 0) ? ErrorMessage.MESSAGE : ErrorMessage.WARNING;
      postMessage(new ErrorMessage(type,"\"" + str + "\" terminated;" +
                                   " exit value: " + exitValue));
    } // end if
    postMessage(new TerminateMessage(exitValue == 0));
  } // end notifyTermination

} // end AbstractExecute

/* ******************************************************************
   end of file
*********************************************************************/
