/* ******************************************************************

   class    : DefaultLinker
   date     : 26.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.applet.Applet;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import AppletView.AppletTagParser;
import AppletView.AppletTag;
import AppletView.AppletTagParsingException;
import CustCtrl.BaseDialog;
import CustCtrl.InsetsPanel;
import Utils.ResourceUtil;
import Messages.*;
import SysExt.FileClassLoader;

/**
 * A "linker" for java applications and applets.  As java classes
 * need not to be linked, this plugin only searches for main
 * methods in classes or applet tags in html files.
 *
 * @author puehringer edgar
 */
public class DefaultLinker extends AbstractLinker {

  // class variable declarations
  /** Key to the compiling progress label resource */
  private static final String LINK_LABEL = "linker.linkLabel";

  // instance variable declarations
  /** The current selected executable file. */
  protected Hashtable curFiles;
  /** The least resently modified applet class file. */
  protected File curApplet;
  /** The class loader for the class file inspection. */
  protected FileClassLoader loader;
  /** The current project path. */
  protected String curProjPath;
  /** The number of inspection errors. */
  protected int errors;
  /** The number of inspection warnings. */
  protected int warnings;
  /** The text for the progress message. */
  private String linkLabel;

  // constructors
  /** Create a new DefaultLinker class. */
  public DefaultLinker(ResourceBundle resources, Properties project) {
    super(resources,project);
    curFiles    = new Hashtable();
    linkLabel   = ResourceUtil.getResStringOrKey(this.resources,LINK_LABEL);
    curProjPath = getProjectDirectory();
  } // end constructor DefaultLinker

  // methods
  public File[] link(Frame frame, File[] files) {
    long timeStamp    = System.currentTimeMillis();
    Hashtable old     = curFiles;
    Vector toInspect  = new Vector();
    Vector executable = new Vector();
    ObjectDesc desc;
    int size;
    String appName;

    curFiles = files.length > 0 ?
               new Hashtable(files.length) :
               new Hashtable();
    appName  = project.getProperty(Const.CUR_APPLET);
    if (appName != null) appName = ResourceUtil.urlToFile(appName);
    curApplet = appName == null ? null : new File(appName);
    errors   = 0;
    warnings = 0;
    // create a vector containing the files to inspect
    for (int i=0; i<files.length; i++) {
      desc = (ObjectDesc)old.get(files[i].getAbsolutePath());
      if ((desc == null) || (files[i].lastModified() > desc.time)) {
        toInspect.addElement(files[i]);
      }
      else {
        curFiles.put(files[i].getAbsolutePath(),desc);
        if (desc.executable) executable.addElement(files[i]);
      } // end if
    } // end for
    // inspect changed or new files
    size = toInspect.size();
    protectEnvironment(true);
    for (int i=0; i<size; i++) {
      File file   = (File)toInspect.elementAt(i);
      String str  = file.getAbsolutePath();
      String path = getProjectDirectory();

      if (str.startsWith(path)) {
        str = str.substring(path.length());
        if (str.startsWith(File.separator)) str = str.substring(1);
      } // end if
      notifyLink(linkLabel,new File(str),i,size);
      desc = isExecutable(file);
      if (desc.executable) executable.addElement(file);
      curFiles.put(file.getAbsolutePath(),desc);
    } // end for
    protectEnvironment(false);
    // create array of executable files
    size = executable.size();
    files = new File[size];
    for (int i=0; i<size; i++) files[i] = (File)executable.elementAt(i);
    // Free the class loader
    loader = null;
    System.gc();
    // report a error summary
    timeStamp = (System.currentTimeMillis() - timeStamp) / 1000;

    String str  = "Inspection done; " + errors + " error(s), " +
                  warnings + " warning(s).";
    String min  = "" + (timeStamp % 60);
    if (min.length() < 2) min = "0" + min;
    String info = "Inspection statistics: " + toInspect.size() +
                  " file(s); time: " + (timeStamp / 60) + ":" + min;
    if (errors > 0) postMessage(new ErrorMessage(ErrorMessage.ERROR,str,info));
      else if (warnings > 0) postMessage(new ErrorMessage(ErrorMessage.WARNING,str,info));
        else postMessage(new ErrorMessage(ErrorMessage.MESSAGE,str,info));
    notifyEndOfLink(errors == 0);
    // write the least recently modified applet to the project properties
    if (curApplet != null) {
      appName = ResourceUtil.fileToURL(curApplet.getAbsolutePath());
      if (!appName.equals(project.getProperty(Const.CUR_APPLET))) {
        project.put(Const.CUR_APPLET,appName);
        setProjectChanged();
      } // end if
    } // end if
    return files;
  } // end link

  private ObjectDesc isExecutable(File file) {
    String path         = file.getAbsolutePath();
    String[] extensions = getExtensions();
    boolean isClass     = false;

    // the first half of the extensions refer to classes, the second half
    // refers to html files
    for (int i=0; i<(extensions.length / 2); i++) {
      if (path.endsWith(extensions[i])) {
        isClass = true;
        break;
      } // end if
    } // end for
    if (isClass) return inspectClass(file);
      else return inspectHTML(file);
  } // isExecutable

  private ObjectDesc inspectClass(File file) {
    ObjectDesc desc = new ObjectDesc(file.lastModified(),false);
    String path     = getProjectDirectory();
    if ((loader == null) || !path.equals(curProjPath)) {
      loader      = new FileClassLoader(path);
      curProjPath = path;
    } // end if
    try {
      Class clazz  = loader.loadClass(file,false);
      Class[] args = new Class[1];
      Method m;
      int modifiers;

      if (((curApplet == null) ||
           (desc.time >= curApplet.lastModified())) &&
          (Applet.class.isAssignableFrom(clazz))) {
        curApplet = file;
      } // end if
      args[0]   = String[].class;
      m         = clazz.getMethod("main",args);
      modifiers = m.getModifiers();
      if (Modifier.isStatic(modifiers)) {
        desc.executable = true;
        return desc;
      } // end if
    } catch (NoSuchMethodException excep) {
      // do nothing
    } catch (ClassNotFoundException excep) {
      desc.time--; // this forces inspection of this file on next link.
      cantLoadError(file);
    } catch (ClassFormatError excep) {
      desc.time--; // this forces inspection of this file on next link.
      cantLoadError(file);
    } catch (NoClassDefFoundError excep) {
      desc.time--; // this forces inspection of this file on next link.
      noSuperClassError(file);
    }
    return desc;
  } // inspectClass

  private final void cantLoadError(File file) {
    String str = relPath(file);

    str = "Can't load \"" + str + "\" with the current project path settings.";
    postMessage(new ErrorMessage(ErrorMessage.WARNING,str));
    warnings++;
  } // end cantLoadError

  private final void noSuperClassError(File file) {
    String str = relPath(file);

    str = "Can't load superclass or interface of \"" + str +
          "\" with the current project path settings.";
    postMessage(new ErrorMessage(ErrorMessage.WARNING,str));
    warnings++;
  } // end noSuperClassError

  private final String relPath(File file) {
    String str  = file.getAbsolutePath();
    String path = getProjectDirectory();

    if (str.startsWith(path)) {
      str = str.substring(path.length());
      if (str.startsWith(File.separator)) str = str.substring(1);
    } // end if
    return str;
  } // end relPath

  private ObjectDesc inspectHTML(File file) {
    ObjectDesc desc        = new ObjectDesc(file.lastModified(),false);
    FileInputStream fin    = null;
    AppletTagParser parser = null;
    AppletTag tag;

    try {
      fin             = new FileInputStream(file);
      parser          = new AppletTagParser(fin);
      tag             = parser.parse();
      desc.executable = (tag != null);
    } catch (AppletTagParsingException excep) {
      ErrorMessage em;

      desc.time--; // this forces inspection of this file on next link.
      em = new LocatableErrorMessage(ErrorMessage.ERROR,excep.getMessage(),
                                     new File(getProjectDirectory()),
                                     file,parser.lineno(),null);
      postMessage(em);
      errors++;
    } catch (IOException excep) {
      desc.time--; // this forces inspection of this file on next link.
      excep.printStackTrace();
    } finally {
      if (fin != null) try { fin.close(); } catch (IOException e){}
    }
    return desc;
  } // end inspectHTML

  // inner classes
  class ObjectDesc {

    // instance variable declarations
    public long time;
    public boolean executable;

    // constructors
    public ObjectDesc(long time, boolean executable) {
      this.time       = time;
      this.executable = executable;
    } // end constructor ObjectDesc

  } // end ObjectDesc

} // end DefaultLinker

/* ******************************************************************
   end of file
*********************************************************************/
