/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : FrameManager
   date       : 30.06.98
   description: this class handles the window events for a logical
                group of frames and provides a frame list.

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class FrameManager extends Observable
  implements FrameContentManager, WindowListener {

  // instance variable declarations
  private Hashtable hashtable;

  // constructors
  public FrameManager() {
    hashtable = new Hashtable();
  } // end constructor FrameManager

  // class methods
  /** Return the hosting frame of a component or null if it doesn't exist. */
  public final static Frame getParent(Component comp) {
    for (Container p = comp.getParent(); p != null; p = p.getParent()) {
      if (p instanceof Frame) return (Frame)p;
    } // end for
    return null;
  } // end getParent

  /**
   * Return the frame who posted the window event e or null, if it
   * was no frame.
   */
  private final static Frame getEventFrame(WindowEvent e) {
    Window w = e.getWindow();

    if (w instanceof Frame) return (Frame)w;
    return null;
  } // end getEventFrame

  // methods
  /**
   * Get a new frame that is registered in this frame manager.
   */
  public Frame getFrame() {
    Frame frame = new Frame();

    register(frame);
    return(frame);
  } // end getFrame

  /**
   * Registers a frame in this frame manager with the default frame
   * manager listener.
   */
  public void register(Frame frame) {
    register(frame,this);
  } // end register

  /**
   * Registers a frame in this frame manager with the specified frame
   * manager listener.
   */
  public void register(Frame frame, FrameContentManager fml) {
    if (fml == null) fml = this;
    if (hashtable.containsKey(frame)) {
      if (fml.equals(hashtable.get(frame))) return;
    }
    else {
      frame.addWindowListener(this);
    } // end if
    addObserver(fml);
    hashtable.put(frame,fml);
    setChanged();
    notifyObservers(getFrames());
  } // end register

  /** Unregisters a frame in this frame manager. */
  public void unregister(Frame frame) {
    FrameContentManager fml;

    if (hashtable.containsKey(frame)) {
      frame.removeWindowListener(this);
      fml = (FrameContentManager)hashtable.get(frame);
      hashtable.remove(frame);
      if (fml != null) deleteObserver(fml);
      setChanged();
      notifyObservers(getFrames());
    } // end if
  } // end unregister

  /**
   * Sets the title of the spezified frame. This method should be used
   * instead of setTitle in class Frame to make it possible to keep track
   * of the frame titles the menues of the single frames.
   */
  public void setTitle(Frame frame, String title) {
    String oldTitle = frame.getTitle();
    frame.setTitle(title);
    if (hashtable.containsKey(frame) &&
        !title.equals(oldTitle)) {
      setChanged();
      notifyObservers(getFrames());
    } // end if
  } // end setTitle

  /**
   * Get a enumeration of all frames in this frame manager.
   */
  public Enumeration getFrames() {
    return hashtable.keys();
  } // end getFrames

  /**
   * Get the number of frames managed by this frame manager.
   */
  public int getFrameCount() {
    return hashtable.size();
  } // end getFrameCount

  /**
   * Dispose all managed frames if all canClose methods return
   * true.
   *
   * @return True if frames were disposed, false otherwise.
   */
  public boolean closeAll() {
    boolean success = true;
    Enumeration e   = hashtable.keys();

    while (e.hasMoreElements() && success) {
      Frame f                 = (Frame)e.nextElement();
      FrameContentManager fmt = (FrameContentManager)hashtable.get(f);

      success = fmt.canClose();
    } // end while
    if (success) {
      e = hashtable.keys();
      while (e.hasMoreElements()) {
        Frame f                 = (Frame)e.nextElement();
        FrameContentManager fmt = (FrameContentManager)hashtable.get(f);

        f.removeWindowListener(this);
        deleteObserver(fmt);
        f.setVisible(false);
        f.dispose();
      } // end while
      hashtable.clear();
    } // end if
    return success;
  } // end closeAll

// ----- FrameContentManager ----------------------------------------

  /** Default implementation of the frame content manager */
  public boolean canClose() {
    return true;
  } // end canClose

  /** Default implementation of the frame content manager */
  public boolean isMainFrame() {
    return false;
  } // end isMainFrame

  /** Default implementation of the frame content manager */
  public void update(Observable o, Object arg) {
  } // end update

// ----- WindowListener ---------------------------------------------

  /** Invoked when a window has been opened. */
  public void windowOpened(WindowEvent e) {}

  /** Invoked when a window is in the process of being closed. */
  public void windowClosing(WindowEvent e) {
    Frame f = getEventFrame(e);

    if (f == null) return;
    if (hashtable.containsKey(f)) {
      FrameContentManager fmt = (FrameContentManager)hashtable.get(f);
      if (fmt.isMainFrame()) {
        closeAll();
        return;
      } else if (!fmt.canClose()) {
        return;
      } // end if
    } // end if
    windowClosed(e);
  } // end windowClosing

  /** Invoked when a window has been closed. */
  public void windowClosed(WindowEvent e) {
    boolean isManaged;
    Frame f = getEventFrame(e);

    if (f == null) return;
    isManaged = hashtable.containsKey(f);
    f.removeWindowListener(this);
    if (isManaged) {
      FrameContentManager fml = (FrameContentManager)hashtable.get(f);

      hashtable.remove(f);
      deleteObserver(fml);
    } // end if
    f.setVisible(false);
    f.dispose();
    if (isManaged) {
      setChanged();
      notifyObservers(getFrames());
    } // end if
  } // end windowClosed

  /** Invoked when a window is iconified. */
  public void windowIconified(WindowEvent e) {}

  /** Invoked when a window is de-iconified. */
  public void windowDeiconified(WindowEvent e) {}

  /** Invoked when a window is activated. */
  public void windowActivated(WindowEvent e) {}

  /** Invoked when a window is deactivated. */
  public void windowDeactivated(WindowEvent e) {}

} // end FrameManager

/* ******************************************************************
   end of file
*********************************************************************/
