/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.6)

   class      : JToolBarLoader
   date       : 24.09.98

 *********************************************************************/

package MenuPreview;

/* ******************************************************************
   import statements
 *********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import com.sun.java.swing.event.*;
import com.sun.java.swing.*;

import Utils.ResourceUtil;

/* ******************************************************************
   top level class declaration
 *********************************************************************/

/**
 * A class that builds up a toolbar by reading the settings from an
 * input stream loaded into a properties object.
 *
 * @author  Puehringer Edgar
 */
class JToolBarLoader {

  // instance variable declarations
  private Properties resources;
  private Hashtable commands;
  private ActionListener defaultAction;
  private String imagePath;

  // constructors
  JToolBarLoader(InputStream is) throws IOException {
    resources = new Properties();
    resources.load(is);
    commands      = null;
    defaultAction = null;
    imagePath     = System.getProperty("user.dir");
  } // end constructor JToolBarLoader

  JToolBarLoader(Properties resources) {
     this.resources = resources;
     commands       = null;
     defaultAction  = null;
    imagePath       = System.getProperty("user.dir");
  } // end constructor JToolBarLoader

  JToolBarLoader(Properties resources, String imagePath) {
     this.resources = resources;
     this.imagePath = imagePath;
     commands       = null;
     defaultAction  = null;
  } // end constructor JToolBarLoader

  // class methods
  /** Filters the mnemonic identifiers out of a label string. */
  private static String filterMnemonic(String label, char[] ch) {
    int index = label.indexOf(Const.MNEMONIC_ESCAPE);

    if (index < 0) return label;
    if ((index + 1) < label.length()) {
      char c = label.charAt(index + 1);

      if (c != Const.MNEMONIC_ESCAPE) ch[0] = c;
      return label.substring(0,index) + c +
        filterMnemonic(label.substring(index + 2),ch);
    }
    else {
      return label.substring(0,index);
    } // end if
  } // end filterMnemonic

  // methods
  /** Return a hashtable containing the toolbar commands. */
  public Hashtable getCommands() {
    return commands;
  } // end getCommands

  /**
   * Set the hashtable containing the toolbar commands.
   * If you use it, this must be called before createToolBar
   *
   * @see #createToolBar
   */
  public void setCommands(Hashtable commands) {
    this.commands = commands;
  } // end setCommands

  /** Return the default action listener. */
  public ActionListener getDefaultActionListener() {
    return defaultAction;
  } // end getDefaultActionListener

  /**
   * Set the default action listener.
   * If you use it, this must be called before createToolBar.
   *
   * @see #createToolBar
   */
  public void setDefaultActionListener(ActionListener al) {
    defaultAction = al;
  } // end setDefaultActionListener

  /**
   * Create a button to go inside of the toolbar.  By default this
   * will load an image resource.
   *
   * @param key The key in the resource file to serve as the basis
   *            of lookups.
   */
  protected JButton createToolBarButton(String key) {
    URL url     = getResource(key + Const.IMAGE_SUFFIX);
    String astr = getResourceString(key + Const.ACTION_SUFFIX);
    String tip  = getResourceString(key + Const.TOOLTIP_SUFFIX);
    JButton b;

    if (url == null) {
      String label = getResourceString(key + Const.LABEL_SUFFIX);

      if (label == null) {
        reportError("Image and label property for toolbar button key \"" +
                  key + "\" not found");
        return null;
      }
      else {
        char[] mnemonic = new char[1];

        mnemonic[0] = (char)0;
        b           = new JButton(filterMnemonic(label,mnemonic)) {

          public float getAlignmentY() {
            return 0.5f;
          } // end getAlignmentY

        };
        if (mnemonic[0] != (char)0)
          b.setMnemonic(mnemonic[0]);
        // end if
      } // end if
    }
    else {
      b = new JButton(loadImageIcon(url)) {

        public float getAlignmentY() {
          return 0.5f;
        } // end getAlignmentY

      };
    } // end if
    b.setRequestFocusEnabled(false);
    b.setMargin(new Insets(1,1,1,1));
    if(tip != null) b.setToolTipText(tip);
    if(astr == null) astr = key;
    b.setActionCommand(astr);

    ActionListener a = getAction(astr);

    if(a != null) {
      b.addActionListener(a);
      if(a instanceof Action) {
        Action action = (Action)a;

        action.addPropertyChangeListener(new ActionChangedListener(b));
        b.setEnabled(action.isEnabled());
      } // end if
      b.setActionCommand(astr);
    }
    else {
      b.setEnabled(false);
    } // end if
    return b;
  } // end createToolBarButton

  /**
   * Return the Action stored in the command hashtable.  If it doesn't
   * exist, return the default action.
   */
  protected ActionListener getAction(String cmd) {
    ActionListener a;

    if(commands == null) a = null;
      else a = (ActionListener)commands.get(cmd);
    if(a == null) return defaultAction;
      else return a;
  } // end getAction

  /**
   * Read a string from the resource file containing the toolbar
   * definition.
   */
  protected String getResourceString(String key) {
    return resources.getProperty(key);
  } // end getResourceString

  protected URL getResource(String key) {
    String name = getResourceString(key);
    String path;
    URL url;

    if(name == null) return null;
    path = ResourceUtil.fileToURL(imagePath);
    if (path.endsWith("/")) path += name;
      else path += ("/" + name);
    try {
      url = new URL(path);
    } catch (MalformedURLException excep) {
      reportError("Malformed URL: " + path);
      url = null;
    }
    return url;
  } // end getResource

  /**
   * Take the given string and chop it up into a series
   * of strings on whitespace boundries. This is useful
   * for trying to get an array of strings out of the
   * resource file.
   */
  protected String[] tokenize(String input) {
    Vector v = new Vector();
    StringTokenizer t = new StringTokenizer(input);
    String cmd[];

    while(t.hasMoreTokens()) v.addElement(t.nextToken());
    cmd = new String[v.size()];
    for(int i = 0;i < cmd.length;i++)
      cmd[i] = (String)v.elementAt(i);
    // end for
    return cmd;
  } // end tokenize

  /**
   * Create the toolbar.  By default this reads the
   * resource file for the definition of the toolbar.
   */
  protected JToolBar createToolBar(String key) {
    JToolBar tb = new JToolBar();
    String tbDesc = getResourceString(key);
    String[] toolKeys;

    if (tbDesc == null) {
      reportError("ToolBar \"" + key + "\" not found or empty");
      return null;
    } // end if
    toolKeys = tokenize(tbDesc);
    for (int i=0; i<toolKeys.length; i++) {
      if(toolKeys[i].equals("-")) {
        tb.add(Box.createHorizontalStrut(Const.STRUT_SIZE));
      }
      else {
        Component comp = createTool(toolKeys[i]);

        if (comp != null) tb.add(comp);
      } // end if
    } // end for
    tb.add(Box.createHorizontalGlue());
    tb.setBackground(SystemColor.control);
    return tb;
  } // end createToolBar

  /** Hook through which every toolbar item is created. */
  protected Component createTool(String key) {
    return createToolBarButton(key);
  } // end createTool

  /** Hook through which ImageIcons are loaded. */
  protected ImageIcon loadImageIcon(URL url) {
    return new ImageIcon(url);
  } // end loadImageIcon

  protected void reportError(String text) {
    System.err.println("JToolBarLoader: " + text + ".");
  } // end reportError

  // inner classes
  private class ActionChangedListener implements PropertyChangeListener {

    // class variable declarations
    JButton button;

    // constructors
    ActionChangedListener(JButton b) {
      super();
      button = b;
    } // end constructor ActionChangedListener

    // methods
    public void propertyChange(PropertyChangeEvent e) {
      String propertyName = e.getPropertyName();

      if(e.getPropertyName().equals(Action.NAME)) {
        String text = (String)e.getNewValue();

        button.setText(text);
      } else if(propertyName.equals("enabled")) {
        Boolean enabledState = (Boolean)e.getNewValue();

        button.setEnabled(enabledState.booleanValue());
      } // end if
    } // end propertyCange

  } // end ActionChangedListener

} // end JToolBarLoader

/* ******************************************************************
   end of file
 *********************************************************************/
