// project: SID - Structured Interactive Documentation
// author : Dipl.Ing. Stefan Chiettini

package SID.Meta;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.net.*;

import Utils.*;
import SID.Utils.*;
//import SID.*; /* disabled by PE */
//import FileServer.*; /* disabled by PE */

import com.sun.java.swing.tree.*;

class Filter implements FilenameFilter{

	private String extension;

	Filter(String ext){
		extension = ext.toUpperCase();
	}

	public boolean accept(File dir, String name){
		File f = new File(dir+name);
		if (f.isDirectory()) return true;

		return name.toUpperCase().endsWith(extension);
	}
}

public abstract class Hierarchy {

	protected Hashtable classes;		// holds ClassNodes
	protected ClassNode root;
	protected String extension;

	protected Hierarchy(ClassInfo rootClass){
		classes = new Hashtable(15);
		root = new ClassNode(rootClass);
		classes.put(new Integer(rootClass.name.hashCode()) , root);
		extension = "";
	}

	// the name has to contain the package name
	// if class is already existing, it is replaced
	public void addClass(ClassInfo ci){
		// put into class register

		ClassNode node = new ClassNode(ci);
		classes.put(new Integer(ci.name.hashCode()) , node);

		// check for superclass
		ClassNode supClass = (ClassNode)classes.get(new Integer(node.thisClass.superName.hashCode()));
		if (supClass != null){
			supClass.addSubClass(node);
			node.addSuperClass(supClass);
		}

		// check for subclasses
		for (Enumeration e = classes.elements(); e.hasMoreElements();){
			ClassNode n = (ClassNode)e.nextElement();
			
			if (n.thisClass.superName.equals(ci.name)){
				n.addSuperClass(node);
				node.addSubClass(n);
			}
		}
	}

	public void removeClass(String name){
		Integer key = new Integer(name.hashCode());
		ClassNode c = (ClassNode)classes.get(key);
		if (c != null){
			c.unchain();
			classes.remove(key);
		}
	}

	public void printHierarchy(ClassNode node, int depth){
		
		for (int i=0; i<depth; i++)	System.out.print("   ");
		System.out.println(node.thisClass.name + " ->" + node.thisClass.fileName);
		
		for (Enumeration e = subClassesOf(node.thisClass.name); e.hasMoreElements();){ //node.subClasses(); e.hasMoreElements();){
			printHierarchy((ClassNode)e.nextElement(), depth+1);
		}
	}

	public void printHierarchy(){ printHierarchy(root, 0); }
	
	// return the class with the exact name (including package name)
	// or the first class matching the name (without considering different packages)
	public ClassInfo findClass(String name){
		ClassNode findClass = (ClassNode)classes.get(new Integer(name.hashCode()));
		if (findClass != null) return findClass.thisClass;

		// try to find the class in one of the packages

		for (Enumeration e = classes.elements(); e.hasMoreElements();){
			ClassNode node = (ClassNode)e.nextElement();
			if (node.thisClass.name.endsWith(name)) return node.thisClass;
		}

		return null;
	}

	public Enumeration subClassesOf(String name){
		ClassNode findClass = (ClassNode)classes.get(new Integer(name.hashCode()));
		if (findClass != null) return findClass.subClasses();
		return null;
	}

	public ClassInfo superClassOf(String name){
		ClassNode findClass = (ClassNode)classes.get(new Integer(name.hashCode()));
		if (findClass != null) return findClass.superClass();
		return null;
	}

	public Enumeration classes(){
		return classes.elements();
	}

	// scan the given file using the target language specific parser
	// add class information to the hashtable
	// it is unnecessary to build up any sub or superclass relations
	// look at method connectInheritanceRelations()
	public abstract void scanFile(String filename);

	public abstract void scanRemoteFile(InputStream in, URL filename);

	// after parsing all files, the sub- and superclass relations are not built up
	// check all classes in the hierarchy and update their sub- and superclass references
	// depending on the target language (Java - packages, C++ - no limits)
	public abstract void connectInheritanceRelations();

	// scans a set of files
	public void scanFiles(Vector files){
		if (files == null) return;
		for (Enumeration e = files.elements(); e.hasMoreElements();){
			scanFile((String)e.nextElement());
		}
	}
/* disabled by PE
	public void scanDirectory(String root, boolean subdirs){
		int count = 0;
		if (root == null) return;
		File rootDir = new File(root);
		if (rootDir.isDirectory()){
			String[] filelist = rootDir.list(new Filter(extension));
			for (int i = 0; i < filelist.length; i++){
				File f = new File(root+filelist[i]);
				if ((subdirs) & (f.isDirectory())) scanDirectory(root+filelist[i]+File.separator, subdirs);
				else{
					System.out.println("scanning file "+root+filelist[i]);
					scanFile(root+filelist[i]);
					count++;
				}
			}
		}
		else System.out.println("no directory :"+root);
		System.out.println(count + " files scanned.");
		System.out.println(classes.size() + " classes in class tree.");
	}
*/
/* disabled by PE
	// open a remote filedialog to scan a set of remote files
	public long scanRemoteDirectory(Frame parent, RemoteFileManager man, boolean subdirs){
		RemoteFileDialog dlg = man.openFileDialog(parent, "open file for scanning.", RemoteFileManager.LOAD, extension);
		dlg.show();
		long a = System.currentTimeMillis();
		if (dlg.wasCanceled()) return a;
		RemoteDirectory remote = dlg.getDirectory();
		scanRemoteDir(remote, subdirs);
		return a;
	}
*/
/* disabled by PE
	private void scanRemoteDir(RemoteDirectory dir, boolean subdirs){
		String[] filelist = dir.getFileList();
		
		try{
			for (int i=0; i<filelist.length;i++){
				System.out.println("scanning file "+filelist[i]);
				DataInputStream in = new DataInputStream(dir.openFile(filelist[i]));
				scanRemoteFile(in, dir.getFileURL(filelist[i]));
			}
		}
		catch(IOException e){ System.out.println(e); System.exit(1); }
		if (subdirs){
			String[] subs = dir.getSubDirs();

			for(int i = 0; i < subs.length; i++){
				dir.changeToDir(subs[i]);
				System.out.println("changing to subdir "+subs[i]);
				scanRemoteDir(dir, subdirs);
				dir.changeToParent();
			}
		}
	}
*/
	public static String getClassName(String fullName){
		try{
			int idx =  fullName.lastIndexOf('.');
			return fullName.substring(idx+1);
		}
		catch (NullPointerException e){return fullName;}
	}

	public static String getPackageName(String fullName){
		try{
			int idx = fullName.lastIndexOf('.');
			if (idx >= 0)
				return fullName.substring(0, idx);
			else
				return fullName;
		}
		catch (NullPointerException e){return fullName;}
	}

	public void load(DataInputStream in) throws IOException{
		tool.resetSerialization();
		extension = in.readUTF();
		loadIntern(in);
		tool.finalizeSerialization();
	}

	private void loadIntern(DataInputStream in) throws IOException{

		int size = in.readInt();
		
		classes = new Hashtable(size);
		Object o = tool.readClassInfo(in);

		for (int i=0; i< size; i++){
			ClassInfo ci=null;
			try{
				ci = (ClassInfo)(o.getClass().newInstance());
			}
			catch (InstantiationException e1) {System.out.println(e1.toString());}
			catch (IllegalAccessException e1) {System.out.println(e1.toString());}
			
			ci.load(in);
			addClass(ci);
		}
	}			

	public void store(DataOutputStream out) throws IOException{
		tool.resetSerialization();

		out.writeUTF(extension);
		out.writeInt(classes.size());

		Enumeration e = classes.elements(); 
		if (e.hasMoreElements()){
			ClassNode cn = (ClassNode)e.nextElement();

			tool.writeClassInfo(cn.thisClass, out);
			cn.thisClass.store(out);
		
			for (;e.hasMoreElements();){
				cn = (ClassNode)e.nextElement();
				cn.thisClass.store(out);
			}
		}
		tool.finalizeSerialization();
	}

	// get class information directly from file without loading the
	// complete hierarchy into main memory
	public static ClassInfo getClassByName(String classname, String filename) throws IOException{
		tool.resetSerialization();

		DataInputStream in = new DataInputStream(new FileInputStream(new File(filename)));

		String extension = in.readUTF();
		int size = in.readInt();
		
		//classes = new Hashtable(size);
		Object o = tool.readClassInfo(in);
		for (int i=0; i< size; i++){
			ClassInfo ci=null;
			try{
				ci = (ClassInfo)(o.getClass().newInstance());
			}
			catch (InstantiationException e1) {e1.printStackTrace();}
			catch (IllegalAccessException e1) {e1.printStackTrace();}
			
			ci.load(in);
			if (ci.name.equals(classname)) return ci;
//			System.out.println(ci.name + " extends " + ci.superName);
		}
		tool.finalizeSerialization();
		return null;
	}

	// build a new Hierarchy from file
	// check if the hierarchy type is supported by the currently active project
/*  public static Hierarchy createHierarchy(DataInputStream in, SIDMain app) throws IOException{
		String ext = in.readUTF();
		Project p = app.getCurrentProject();
		if (p.targetExtension.equals(ext)){
			Hierarchy h = p.createNewHierarchy();
			tool.resetSerialization();
			h.loadIntern(in);
			tool.finalizeSerialization();
		}
		else{
			String s = "The meta information has language '"+ConfigurationEditor.getLanguageByExtension(ext)+"'.\n";
			s += "The active project has language '"+ConfigurationEditor.getLanguageByExtension(p.targetExtension)+"'.";
			MessageBox.Box(app.getMainFrame(), s, "S.I.D.", MessageBox.EXCLAMATION);
		}
		return null;
  } */ /* disabled by PE */

	// build the flat tree node depending of used target language
	public abstract DefaultMutableTreeNode getFlatClassTree(DefaultMutableTreeNode root, boolean packed);

	protected abstract DefaultMutableTreeNode buildClassNode(ClassInfo ci, boolean packed);

	private boolean hasUnmarkedClasses(){
		for (Enumeration e=classes.elements(); e.hasMoreElements();){
			ClassNode n = (ClassNode)e.nextElement();
			if (!n.mark) return true;
		}
		return false;
	}

	// find an unmarked class which has a superclass not in the hierarchy
	private ClassNode getIndependantClass(){
		for (Enumeration e=classes.elements(); e.hasMoreElements();){
			ClassNode n = (ClassNode)e.nextElement();
			if ((!n.mark) && (n.superClass() == null)) return n;
		}
		return null;
	}

	public DefaultMutableTreeNode getInheritanceClassTree(){
		// reset the marks
		for (Enumeration e=classes.elements(); e.hasMoreElements();){
			ClassNode n = (ClassNode)e.nextElement();
			n.mark = false;
		}
		DefaultMutableTreeNode first = new DefaultMutableTreeNode("classes", true);
		first.add(getInheritanceClassTree(root));		// first try
		while (hasUnmarkedClasses()){
			ClassNode indep = getIndependantClass();
			if (indep == null) break;					// should never occur
			first.add(getInheritanceClassTree(indep));
		}

		return first;
	}

	// build a tree representing the inheritance hierarchy of the given subtree
	private DefaultMutableTreeNode getInheritanceClassTree(ClassNode node){
		DefaultMutableTreeNode root = buildClassNode(node.thisClass, true);
		node.mark = true;
		for (Enumeration e = node.subClasses(); e.hasMoreElements();){
			root.add(getInheritanceClassTree((ClassNode)e.nextElement()));
		}
		return root;
	}

	public DefaultMutableTreeNode getClassList(DefaultMutableTreeNode root, boolean packed){
		for (Enumeration e = classes(); e.hasMoreElements();){
			ClassInfo ci = (ClassInfo)((ClassNode)e.nextElement()).thisClass;
			DefaultMutableTreeNode classNode = buildClassNode(ci, packed);
			
			// add class node sorted by class name
			int idx = 0;
			Enumeration children = root.children();
			boolean found = false;
			
			while (children.hasMoreElements() && !found){
				DefaultMutableTreeNode cur = (DefaultMutableTreeNode)children.nextElement();
				String curName = (String)((ClassInfo)cur.getUserObject()).name;
				if (curName.compareTo(ci.name) > 0){
					root.insert(classNode, idx);
					found = true;
				}
				idx++;
			}
			if (!found) root.add(classNode);
		}
		
		return root;
	}

	public DefaultMutableTreeNode getClassesPerFile(DefaultMutableTreeNode root, String fileName, boolean packed){	
		for (Enumeration e = classes(); e.hasMoreElements();){
			ClassInfo ci = (ClassInfo)((ClassNode)e.nextElement()).thisClass;
			if (ci.fileName.endsWith(fileName)){
				DefaultMutableTreeNode classNode = buildClassNode(ci, packed);
			
				// add class node sorted by class name
				int idx = 0;
				Enumeration children = root.children();
				boolean found = false;
			
				while (children.hasMoreElements() && !found){
					DefaultMutableTreeNode cur = (DefaultMutableTreeNode)children.nextElement();
					String curName = (String)((ClassInfo)cur.getUserObject()).name;
					if (curName.compareTo(ci.name) > 0){
						root.insert(classNode, idx);
						found = true;
					}
					idx++;
				}
				if (!found) root.add(classNode);
			}
		}
		return root;
	}
}
