/* ******************************************************************

   class    : ANSIPreprocFilter
   date     : 04.08.98
   compiler : javac (jdk 1.1.6)

*********************************************************************/

package Streams;

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.*;

/**
 *
 * A reader that replaces three character sequences and merges lines
 * like described in the ANSI standard for C preprocessors.
 *
 * @author puehringer edgar
 *
 */
public class ANSIPreprocFilter extends PushbackReader {

  // constructors
  /**
   * Constructs a new ANSIPreprocFilter initialized with the
   * specified reader.
   *
   * @param in The reader.
   */
  public ANSIPreprocFilter(Reader in) {
    super(in);
  } // end constructor ANSIPreprocFilter

  // class methods
  /** Only for debuging purposes */
  public static void main(String[] args) throws IOException {
    ANSIPreprocFilter filter = new ANSIPreprocFilter(new InputStreamReader(System.in));
    int c;

    c = filter.read();
    while (c != -1) {
      System.out.print(""+(char)c);
      c = filter.read();
    } // end while
    filter.close();
  } // end main

  // methods
  /**
   * Reads a character.  The method will block if no input is available.
   *
   * @return The character read, or -1 if the end of the stream is reached.
   */
  public int read() throws IOException {
    // This is the routine that really implements the special
    // functionality of this class; the others just call this
    // one to get the data that they need.
    int c = super.read();

    switch (c) {
      case (int)'?':
        c = super.read();
        if (c != (int)'?') {
          super.unread(c);
          return (int)'?';
        } // end if
        c = super.read();
        switch (c) {
          case (int)'=':  return '#';
          case (int)'/':  return '\\';
          case (int)'\'': return '^';
          case (int)'(':  return '[';
          case (int)')':  return ']';
          case (int)'!':  return '|';
          case (int)'<':  return '{';
          case (int)'>':  return '}';
          case (int)'-':  return '~';
          default:
            super.unread(c);
            super.unread((int)'?');
            return (int)'?';
        } // end switch
      case (int)'\\':
        c = super.read();
        switch (c) {
          case (int)'\r':
            c = super.read();
            if (c != '\n') super.unread(c);
            return read();
          case (int)'\n':
            return read();
          default:
            super.unread(c);
            return '\\';
        } // end switch
      default:
        return c;
    } // end switch
  } // end read

  /**
   * Reads into an array of characters.
   * Blocks until some input is available.
   *
   * @param cbuf The buffer into which the data is read.
   * @param off The start offset of the data.
   * @param len The maximum number of bytes read.
   * @return The actual number of characters read, -1 is
   *         returned when the end of the stream is reached.
   */
  public int read(char cbuf[], int off, int len) throws IOException {
    for (int i=off; i<(off + len); i++) {
      int c = read();

      if (c == -1) return (i - off) == 0 ? -1 : i - off;
      cbuf[i] = (char)c;
    } // end for
    return len;
  } // end read

  /**
   * Skips bytes of input.
   *
   * @param n         bytes to be skipped
   * @return  actual number of bytes skipped
   */
  public long skip(long n) throws IOException {
    // Can't just read n bytes from 'in' and throw them
    // away, because n bytes from 'in' doesn't necessarily
    // correspond to n bytes from 'this'.
    for (int i=1; i <= n; i++) {
      int c = read();

      if (c == -1) return i - 1;
    } // end for
    return n;
  } // end skip

  /**
   * Returns the number of bytes that can be read without blocking.
   *
   * @return The number of available bytes.
   */
  public int available() throws IOException {
    // We don't really know.  We can ask 'in', but some of those bytes
    // are probably backslashes followed by line separators, and it's
    // possible that all of them are. So we have to be conservative
    // and return zero.
    return 0;
  } // end available

  /**
   * Push back a single character.
   *
   * @param c The character to push back.
   */
  public void unread(int c) throws IOException {
    int tmp;

    switch (c) {
      case (int)'#':  tmp = (int)'=';  break;
      case (int)'\\': tmp = (int)'/';  break;
      case (int)'^':  tmp = (int)'\''; break;
      case (int)'[':  tmp = (int)'(';  break;
      case (int)']':  tmp = (int)')';  break;
      case (int)'{':  tmp = (int)'<';  break;
      case (int)'}':  tmp = (int)'>';  break;
      case (int)'~':  tmp = (int)'-';  break;
      default:
        super.unread(c);
        return;
    } // end switch;
    super.unread((int)'?');
    super.unread((int)'?');
    super.unread(tmp);
  } // end unread

  /**
   * Push back a portion of an array of characters by copying it to the
   * front of the pushback buffer. After this method turns, the next
   * character to be read will have the value cbuf[off], the character
   * after that will have the value cbuf[off+1], and so forth.
   *
   * @param cbuf Character array.
   * @param off Offset of first character to push back.
   * @param len Number of characters to push back.
   */
  public void unread(char cbuf[], int off, int len) throws IOException {
    for (int i=off; i<(off + len); i++) unread((int)cbuf[i]);
  } // end unread

  /**
   * Push back an array of characters by copying it to the front of
   * the pushback buffer. After this method returns, the next character
   * to be read will have the value cbuf[0], the character after that
   * will have the value cbuf[1], and so forth.
   *
   * @param cbuf Character array to push back.
   */
  public void unread(char cbuf[]) throws IOException {
    for (int i=0; i<cbuf.length; i++) unread((int)cbuf[i]);
  } // end unread

} // end ANSIPreprocFilter

/* ******************************************************************
   end of file
*********************************************************************/
