/* ******************************************************************

   class    : FileClassLoader
   date     : Okt 14 1998
   compiler : javac (1.1.6)

******************************************************************* */

package SysExt;

/* ******************************************************************
   import statements
******************************************************************* */

import java.io.*;
import java.util.*;

/**
 * A class loader that loads classes from file references and allows
 * to include another directory into the class path (of this
 * class loader).  This class can't handle jar files.
 *
 * @author puehringer edgar
 */
public class FileClassLoader extends ClassLoader {

  // instance variable declarations
  private Hashtable cache = new Hashtable();
  private String[] aPath;

  // constructors
  /** Create a new FileClassLoader object. */
  public FileClassLoader() {
    this(null);
  } // end constructor FileClassLoader

  /**
   * Create a new FileClassLoader object.
   *
   * @param path A additional path to add to the class loader.
   */
  public FileClassLoader(String path) {
    super();
    Vector vector       = new Vector();
    String str          = System.getProperty("java.class.path");
    StringTokenizer tok = new StringTokenizer(str,File.pathSeparator,false);
    File file;

    if (path != null) {
      file = new File(path);
      if (file.isDirectory()) try {
        vector.addElement(file.getCanonicalPath());
      } catch (IOException excep) {
        vector.addElement(file.getAbsolutePath());
      }
    } // end if
    while (tok.hasMoreTokens()) {
      file = new File(tok.nextToken());
      if (file.isDirectory()) try {
        vector.addElement(file.getCanonicalPath());
      } catch (IOException excep) {
        vector.addElement(file.getAbsolutePath());
      }
    } // end while
    aPath = new String[vector.size()];
    for (int i=0; i<vector.size(); i++) {
      aPath[i] = (String)vector.elementAt(i);
    } // end for
  } // end constructor FileClassLoader

  // methods
  /** Returns a class name for a file name. */
  private final String resolveName(String name) {
    if ((name == null) || !name.endsWith(".class")) return null;
    // clip .class ending
    name = name.substring(0,name.length() - 6);
    for (int i=0; i<aPath.length; i++) {
      if (name.startsWith(aPath[i])) {
        name = name.substring(aPath[i].length());
        if (name.startsWith(File.separator)) name = name.substring(1);
        return name.replace(File.separatorChar,'.');
      } // end if
    } // end for
    return null;
  } // end resolveName

  /** Returns the target path of a class. */
  private final String getTargetPath(String name) {
    File file;

    name = name.replace('.',File.separatorChar) + ".class";
    for (int i=0; i<aPath.length; i++) {
      file = new File(aPath[i],name);
      if (file.exists()) return file.getAbsolutePath();
    } // end for
    return null;
  } // end getTargetPath

  /** Load the binary data of a class from a file specified by name. */
  private final byte[] loadClassData(String name) {
    FileInputStream fis = null;
    byte[] buff         = new byte[4096];
    ByteArrayOutputStream baos;
    int nch;

    try {
      fis  = new FileInputStream(name);
      buff = new byte[4096];
      baos = new ByteArrayOutputStream();
      while ((nch = fis.read(buff,0,buff.length)) != -1) {
        baos.write(buff,0,nch);
      } // end while
      return baos.toByteArray();
    } catch (IOException excep) {
    } finally {
      if (fis != null) try { fis.close(); } catch (IOException e) {}
    }
    return null;
  } // end loadClassData

  /** Load a class from a file. */
  public synchronized Class loadClass(File file, boolean resolve)
    throws ClassNotFoundException {
    String fullName = resolveName(file.getAbsolutePath());

    if (fullName == null) throw new ClassNotFoundException();
    return loadClass(fullName,resolve);
  } // end loadClass

  /** Load a class from a byte buffer. */
  public synchronized Class loadClass(String name, byte[] data, boolean resolve)
    throws ClassNotFoundException {
    Class c = (Class)cache.get(name);

    if (c == null) {
      c = defineClass(name,data,0,data.length);
      cache.put(name,c);
    } // end if
    if (resolve) resolveClass(c);
    return c;
  } // end loadClass

  /** Load the class specified by name. */
  protected synchronized Class loadClass(String name, boolean resolve)
    throws ClassNotFoundException {
    Class c = (Class)cache.get(name);

    if (c == null) {
      String path = getTargetPath(name);

      if (path == null) {
        c = Class.forName(name);
      }
      else {
        byte[] data = loadClassData(path);

        c = defineClass(name,data,0,data.length);
      } // end if
      cache.put(name,c);
    } // end if
    if (resolve) resolveClass(c);
    return c;
  } // end loadClass

} // end FileClassLoader

/* ******************************************************************
   end of file
******************************************************************* */
